#!/usr/bin/perl -w

use strict;

my $VERSION=0.4;

my $specname='';
unless (@ARGV) {
    print "usage: spec2make.pl specname.spec > GNUmakefile

spec2make.pl generates GNUmakefile from spec.
It suppose that it runs in catalog generated by srpm2dir.sh.

If you want to build packages inside the current directory, add 
LOCALBUILD=true
in file local.mak or ../global.mak .

spec2make.pl supports compression of uncompressed source trees.
To help spec2make.pl you should place uncompressed source tree 
corresponding to SourceN in directory UNCOMPRESSED[N][:path:to:skip]
if N is omitted it is assumed to be 0. 
Optional [:path:to:skip] tells spec2make.pl the directory tree level 
where should be the root directory of the archive.
if \%setup macro is used for the corrseponding uncompressed source tree,
spec2make.pl tries to put the root directory of the archive into 
a temporary directory with name guessed from arguments of \%setup macro. 
Otherwise source tree is compressed as is.
";
    exit;
} else {
    $specname=$ARGV[0];
    $specname=$1 if $ARGV[0]=~/^(?:.*\/)?([^\/]+)\.spec$/;
}

my %macro;
$macro{'homedir'}=$ENV{'HOME'};
$macro{'_topdir'}="$ENV{'HOME'}/src/RPM";
$macro{'_topsrcdir'}='%_topdir';
$macro{'_sourcedir'}="$macro{'_topdir'}/SOURCES";
$macro{'_specdir'}="$macro{'_topdir'}/SPECS";
open (RPMMACROS, $ENV{'HOME'}.'/.rpmmacros') || die "can't open .rpmmacros:$!";
while (<RPMMACROS>) {
    if (/^[%]([\w_\/.\d]+)\s+(.+)\s*$/) {
	$macro{$1}=$2 ;
	&macro_subst(\$macro{$1});
    }
}
close (RPMMACROS);

my @source;
my @patch;
my @setup_n;
my @is_setup;
while (<>) {
    chomp;
    $macro{'name'}=$1 if /^Name:\s*(.+)$/;
    $macro{'version'}=$1 if /^Version:\s*(.+)$/;
    $macro{'release'}=$1 if /^Release:\s*(.+)$/;
    $macro{$1}=$2 if /^[%]define\s+([\w_\d]+)\s+([\w_\d.]+)\s*$/;
    $source[$1 ? $1 : 0]=$3 if /^Source(\d*):\s*(.+\/)?([^\/]+)$/;
    $patch[$1 ? $1 : 0]=$3 if /^Patch(\d*):\s*(.+\/)?([^\/]+)$/;
    $setup_n[$1 ? $1 : 0] =$3 if /^[%]setup(\d*)\s*(.+)?-n\s+(\S+)/;
    $is_setup[$1 ? $1 : 0] =1 if /^[%]setup(\d*)/;
}

my @require=grep {$_} @source, @patch;

foreach my $dep (@require) {
    &macro_subst(\$dep);
}
#print join("\n",@require),"\n";
#print join("\n",keys %macro),"\n";
my $depends=join(' ',@require);
my @clean=grep {/^(.+)\.(gz|bz2)$/ and -e $1} @require;

&macro_subst(\$macro{'_sourcedir'});
&macro_subst(\$macro{'_specdir'});

print "
# GNUmakefile; generated from $specname.spec at ".`date`,"
NAME=$specname
PACKAGE=$macro{'name'}
VERSION=$macro{'version'}
RELEASE=$macro{'release'}
TOPDIR=$macro{'_topdir'}
SPECDIR=$macro{'_specdir'}
SOURCEDIR=$macro{'_sourcedir'}

LOCALSOURCEDIR=SOURCES
LOCALSPECDIR=.
SOURCES=$depends
",'
CURDIR=$(shell pwd)
HASHERDIR=$(CURDIR)/HASHER

-include ../global.mak
-include local.mak

ifeq ($(LOCALBUILD),true)
TOPDIR=$(shell pwd)
#SPECDIR=$(TOPDIR)/SPECS
SPECDIR=$(TOPDIR)
SOURCEDIR=$(TOPDIR)/SOURCES
RPMOPTS= --define "_topdir $(TOPDIR)" --define "_tmpdir $(TOPDIR)/TMP"  --define "_specdir $(SPECDIR)" --define "_sourcedir $(SOURCEDIR)"
endif

installed_sources=$(addprefix $(SOURCEDIR)/,$(SOURCES))
local_sources=$(addprefix $(LOCALSOURCEDIR)/,$(SOURCES))
installed_spec=$(SPECDIR)/$(NAME).spec
installed_files=$(installed_sources) $(installed_spec) GNUmakefile
SPECFILE=$(LOCALSPECDIR)/$(NAME).spec

spec2make_utility=$(shell if test -e ../spec2make.pl; then echo ../spec2make.pl; else if test -e "'.$0.'"; then echo "'.$0.'"; else if test -e /usr/bin/spec2make.pl; then echo /usr/bin/spec2make.pl; fi; fi; fi)

all: $(installed_files)
	rpmbuild -ba $(RPMOPTS) $(installed_spec)

echo:
	@echo RPMOPTS=$(RPMOPTS)
	@echo SPECDIR=$(SPECDIR)
	@echo NAME=$(NAME)
	@echo installed_spec=$(installed_spec)

ba: all

bp: $(installed_files)
	rpmbuild -bp $(RPMOPTS) $(installed_spec)

binary: $(installed_files)
	rpmbuild -bb $(RPMOPTS) $(installed_spec)

source: $(installed_files)
	rpmbuild -bs $(RPMOPTS) $(installed_spec)

$(TOPDIR)/SRPMS/$(PACKAGE)-$(VERSION)-$(RELEASE).src.rpm: $(installed_files)
	rpmbuild -bs $(RPMOPTS) $(installed_spec)

$(SOURCEDIR)/%: $(LOCALSOURCEDIR)/%
	@mkdir -p $(SOURCEDIR)
	rm -f $@
	cp -pf $< $@

$(installed_spec): $(SPECFILE)
	@mkdir -p $(SPECDIR)
	cp -pf $(SPECFILE) $(SPECDIR)

ifneq ($(LOCALBUILD),true)
rpm: $(installed_files)
	rpmbuild -bb --clean --rmsource $(installed_spec)

src: $(installed_files)
	rpmbuild -bs --clean --rmsource $(installed_spec)
endif

hsh: $(installed_files) $(TOPDIR)/SRPMS/$(PACKAGE)-$(VERSION)-$(RELEASE).src.rpm
	mkdir -p $(HASHERDIR)
	hsh $(HASHERDIR) $(TOPDIR)/SRPMS/$(PACKAGE)-$(VERSION)-$(RELEASE).src.rpm

GNUmakefile: $(SPECFILE) $(spec2make_utility)
	if [ -e ../spec2make.pl ] ; then perl ../spec2make.pl $(SPECFILE) >$@; \
	else if [ -e '.$0.' ] ; then perl '.$0.' $(SPECFILE) >$@; \
	else spec2make.pl $(SPECFILE) >$@; fi; fi

clean:
	rm -rf *~ '.join(' ',@clean).' \
	 $(addsuffix .gz,$(wildcard *)) \
	 $(addsuffix .bz2,$(wildcard *))
	rm -Rf TMP RPMS SRPMS BUILD
	$(if $(TAR), rm -f $(TAR))

%.bz2: %
	# preserving attrs
	bzip -kf $^

%.gz: %
	# preserving attrs
	cp -p $^ $^.orig
	gzip -9 -f $^
	mv $^.orig $^

.PHONY: binary source clean hsh ba bp

';

#######################################################
# support for uncompressed trees
#######################################################

my @tarball;
foreach my $dir (glob "UNCOMPRESSED*") {
    if (-d $dir) {
	$dir=~/^UNCOMPRESSED(\d*)(.*)/;
	my $sourcenum=$1;
	my $path=$2;
	$sourcenum||=0;
	$path||='';
	$path=~s/^://;
	$path=~s!:!/!g;
	my $tarflag='';
	&macro_subst(\$source[$sourcenum]);
	$tarflag='z' if $source[$sourcenum]=~/\.gz$/ or $source[$sourcenum]=~/\.tgz$/;
	$tarflag='j' if $source[$sourcenum]=~/\.bz2$/;
	my $dirname='';
	if ($is_setup[$sourcenum]) {
	    $dirname="$macro{'name'}-$macro{'version'}";
	    if ($setup_n[$sourcenum]) {
		&macro_subst(\$setup_n[$sourcenum]);
		$dirname=$setup_n[$sourcenum];
	    }
	}
	if ($source[$sourcenum]) {
	    if ($dirname) {
		# %setup macro is used
		print "
$source[$sourcenum]: 
	mkdir -p TMP
	mv $dir/$path TMP/$dirname
	tar c${tarflag}f $source[$sourcenum] -C \$(CURDIR)/TMP $dirname
	mv TMP/$dirname $dir/$path

";
	    } else {
		# raw source; no %setup macro found
		print "
$source[$sourcenum]: 
	tar c${tarflag}f $source[$sourcenum] -C $dir/$path

";
	    }
	    push @tarball, $source[$sourcenum];
	} else {
	    warn "source of folder $dir not found in $specname.spec\n";
	}
    }
}

print "
tarball: ".join(' ',@tarball)."

" if @tarball;


sub macro_subst {
    my $argptr=shift;
    while ($$argptr=~/\%\{?(\w+)\}?/ and $macro{$1}) {
	$$argptr=~s/\%\{?(\w+)\}?/$macro{$1}/;
    }
}

__END__
# File: spec2make.pl
# Author: Igor Vlasenko <vlasenko@imath.kiev.ua>
# Created: Sun Oct 23 22:06:35 2005
# $Id: newfile-template.el,v 1.4 2005/05/26 20:10:12 igor Exp $

#  Local Variables:
#  mode: perl
#  coding: cp1251
#  End:

# rider@ makefile
# -----------------------------------------------
TOPDIR=$(shell pwd)
PACKAGE=$(shell basename $(TOPDIR))
VERSION=$(shell rpm -q --specfile --qf "%{VERSION}\n"  ${TOPDIR}/SPECS/${PACKAGE}.spec|head -1)
UNCOMPRESSED=$(shell test -d UNCOMPRESSED && echo true)

ifeq ($(UNCOMPRESSED),true)
TAR=SOURCES/$(PACKAGE)-$(VERSION).tar.bz2
CLEANFILES=$(TAR)
endif

all: $(TAR)
	rpmbuild -ba --define "_topdir $(TOPDIR)" --define "_tmpdir $(TOPDIR)/TMP"  SPECS/$(PACKAGE).spec

ba: all

bp:
	rpmbuild -bp --define "_topdir $(TOPDIR)" --define "_tmpdir $(TOPDIR)/TMP"  SPECS/$(PACKAGE).spec

clean:
	rm -Rf TMP RPMS SRPMS BUILD
	$(if $(TAR), rm -f $(TAR))

binary: $(TAR)
	rpmbuild -bb --define "_topdir $(TOPDIR)" --define "_tmpdir $(TOPDIR)/TMP"  SPECS/$(PACKAGE).spec

source: $(TAR)
	rpmbuild -bs --define "_topdir $(TOPDIR)" --define "_tmpdir $(TOPDIR)/TMP"  SPECS/$(PACKAGE).spec

ifeq ($(UNCOMPRESSED),true)
$(TAR):
	rm -Rf TMP/export
	mkdir -p TMP/export
	URL=`svn info UNCOMPRESSED|grep URL|cut -f2 -d' '`;\
	svn export $$URL TMP/export/$(PACKAGE)-$(VERSION)
	pushd TMP/export/;tar cvfj $(TOPDIR)/SOURCES/$(PACKAGE)-$(VERSION).tar.bz2 $(PACKAGE)-$(VERSION);popd
tarball: $(TAR)
endif
